]> Pileus Git - wmpus/blobdiff - wm-wmii.c
Rename mode_t to layout_t
[wmpus] / wm-wmii.c
index a2be22d69a4520f15db4fc52006fe841a284cbf3..094f3d1d362eb576b139669425f0fb627e0fab79 100644 (file)
--- a/wm-wmii.c
+++ b/wm-wmii.c
@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, Andy Spencer <andy753421@gmail.com>
+ * Copyright (c) 2011-2012, Andy Spencer <andy753421@gmail.com>
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -35,7 +35,7 @@ typedef enum {
 
 typedef enum {
        SPLIT, STACK, FULL, TAB
-} mode_t;
+} layout_t;
 
 typedef enum {
        TILING, FLOATING
@@ -45,44 +45,44 @@ typedef enum {
 struct win_wm { };
 
 typedef struct {
-       win_t  *win;     // the window
-       int     height;  // win height in _this_ tag
-       state_t state;   // state of window
+       win_t   *win;     // the window
+       int      height;  // win height in _this_ tag
+       state_t  state;   // state of window
 } row_t;
 
 typedef struct {
-       list_t *rows;    // of row_t
-       row_t  *row;     // focused row
-       int     width;   // column width
-       mode_t  mode;    // display mode
+       list_t  *rows;    // of row_t
+       row_t   *row;     // focused row
+       int      width;   // column width
+       layout_t layout;  // column layout
 } col_t;
 
 typedef struct {
-       win_t *win;      // the window
-       int x, y, w, h;  // position of window (in this tag)
-       state_t state;   // state of window
+       win_t   *win;     // the window
+       int x, y, w, h;   // position of window (in this tag)
+       state_t  state;   // state of window
 } flt_t;
 
 typedef struct {
-       list_t *cols;    // of col_t
-       col_t  *col;     // focused col
-       list_t *flts;    // of flt_t
-       flt_t  *flt;     // focused flt
-       layer_t layer;   // focused layer
-       win_t  *geom;    // display size and position
+       list_t  *cols;    // of col_t
+       col_t   *col;     // focused col
+       list_t  *flts;    // of flt_t
+       flt_t   *flt;     // focused flt
+       layer_t  layer;   // focused layer
+       win_t   *geom;    // display size and position
 } dpy_t;
 
 typedef struct {
-       list_t *dpys;    // of dpy_t
-       dpy_t  *dpy;     // focused dpy
-       int     name;    // tag name
+       list_t  *dpys;    // of dpy_t
+       dpy_t   *dpy;     // focused dpy
+       int      name;    // tag name
 } tag_t;
 
 typedef struct {
-       list_t *tags;    // of tag_t
-       tag_t  *tag;     // focused tag
-       win_t  *root;    // root/background window
-       list_t *screens; // display geometry
+       list_t  *tags;    // of tag_t
+       tag_t   *tag;     // focused tag
+       win_t   *root;    // root/background window
+       list_t  *screens; // display geometry
 } wm_t;
 
 #define WIN(node) ((win_t*)(node)->data)
@@ -186,16 +186,16 @@ static int search(tag_t *tag, win_t *target,
        return -1;
 }
 
-/* Set the mode for the windows column in the current tag */
-static void set_mode(win_t *win, mode_t mode)
+/* Set the layout for the windows column in the current tag */
+static void set_mode(win_t *win, layout_t layout)
 {
        col_t *col;
        if (TILING != search(wm_tag, win, NULL, &col, NULL, NULL))
                return;
        printf("set_mode: %p, %d -> %d\n",
-                       col, col->mode, mode);
-       col->mode = mode;
-       if (col->mode == SPLIT)
+                       col, col->layout, layout);
+       col->layout = layout;
+       if (col->layout == SPLIT)
                for (list_t *cur = col->rows; cur; cur = cur->next) {
                        row_t *row = cur->data;
                        row->height = wm_dpy->geom->h;
@@ -212,16 +212,6 @@ static void set_focus(win_t *win)
                return;
        }
 
-       /* - Only grab mouse button on unfocused window,
-        *   this prevents stealing all mouse clicks from client windows,
-        * - A better way may be to re-send mouse clicks to client windows
-        *   using the return value from wm_handle_key */
-       for (int i = EV_MOUSE1; i < EV_MOUSE7; i++) {
-               if (wm_focus)
-                       sys_watch(wm_focus, i, MOD());
-               sys_unwatch(win, i, MOD());
-       }
-
        dpy_t *dpy; col_t *col; row_t *row; flt_t *flt;
        switch (search(wm_tag, win, &dpy, &col, &row, &flt)) {
        case TILING:
@@ -279,7 +269,7 @@ static void print_txt(void)
                col_t *col = lcol->data;
                printf("    col:   <%-9p [%p->%p] >%-9p r=%-9p - %dpx @ %d\n",
                                lcol->prev, lcol, lcol->data, lcol->next,
-                               col->row, col->width, col->mode);
+                               col->row, col->width, col->layout);
        for (list_t *lrow = col->rows; lrow; lrow = lrow->next) {
                row_t *row = lrow->data;
                win_t *win = row->win;
@@ -481,7 +471,7 @@ static void shift_focus(int cols, int rows)
                        return;
                row_t *next = get_next(row, rows > 0)->data;
                set_focus(next->win);
-               if (COL(col)->mode != SPLIT)
+               if (COL(col)->layout != SPLIT)
                        wm_update();
        }
        if (cols != 0) {
@@ -646,7 +636,7 @@ static void wm_update_cols(dpy_t *dpy)
                        win->h = ROW(lrow)->height;
                        state_t state = ST_SHOW;
                        int height = 0;
-                       switch (col->mode) {
+                       switch (col->layout) {
                        case SPLIT:
                                sys_move(win, x+margin, y+margin,
                                        col->width, win->h * ((float)my / ty));
@@ -688,6 +678,15 @@ static void wm_update_cols(dpy_t *dpy)
  *******************************/
 void wm_update(void)
 {
+       /* Updates window sizes */
+       for (list_t *ldpy = wm_tag->dpys; ldpy; ldpy = ldpy->next)
+               wm_update_cols(ldpy->data);
+       tag_foreach_flt(wm_tag, ldpy, lflt, win) {
+               flt_t *flt = lflt->data;
+               sys_move(win, flt->x, flt->y, flt->w, flt->h);
+               sys_raise(flt->win);
+       }
+
        /* Show/hide tags */
        tag_foreach_col(wm_tag, dpy, col, row, win)
                sys_show(win, ROW(row)->state);
@@ -701,14 +700,7 @@ void wm_update(void)
                                        sys_show(win, ST_HIDE);
                }
 
-       /* Refresh the display */
-       for (list_t *ldpy = wm_tag->dpys; ldpy; ldpy = ldpy->next)
-               wm_update_cols(ldpy->data);
-       tag_foreach_flt(wm_tag, ldpy, lflt, win) {
-               flt_t *flt = lflt->data;
-               sys_move(win, flt->x, flt->y, flt->w, flt->h);
-               sys_raise(flt->win);
-       }
+       /* Set focused window */
        if (wm_focus)
                set_focus(wm_focus);
 }
@@ -723,10 +715,10 @@ int wm_handle_event(win_t *win, event_t ev, mod_t mod, ptr_t ptr)
        //      mod.shift ? 's' : '-',
        //      mod.win   ? 'w' : '-');
 
-       /* Mouse movement */
-       if (ev == EV_MOUSE1)
-               raise_float(win);
+       /* Mouse events */
        if (EV_MOUSE0 <= ev && ev <= EV_MOUSE7) {
+               if (ev == EV_MOUSE1 && !mod.MODKEY && !mod.up)
+                       return raise_float(win),         0;
                if (ev == EV_MOUSE1 && mod.MODKEY && !mod.up)
                        return set_move(win,ptr,MOVE),   1;
                if (ev == EV_MOUSE3 && mod.MODKEY && !mod.up)
@@ -755,6 +747,9 @@ int wm_handle_event(win_t *win, event_t ev, mod_t mod, ptr_t ptr)
                if (ev == EV_F6) return print_txt(),    1;
                if (ev == 'q')   return sys_exit(),     1;
        }
+       if (mod.MODKEY && mod.shift) {
+               if (ev == 'c')   return sys_show(win, ST_CLOSE), 1;
+       }
 
        /* Floating layer */
        if (ev == ' ') {
@@ -784,7 +779,7 @@ int wm_handle_event(win_t *win, event_t ev, mod_t mod, ptr_t ptr)
                }
        }
 
/* Column mode commands */
        if (mod.MODKEY) {
                switch (ev) {
                case 'd': return set_mode(win, SPLIT), 1;
@@ -809,9 +804,6 @@ int wm_handle_event(win_t *win, event_t ev, mod_t mod, ptr_t ptr)
        if (ev == EV_ENTER && win->state != ST_SHADE)
                return set_focus(win), 1;
 
-       if (EV_MOUSE0 <= ev && ev <= EV_MOUSE7)
-               return set_focus(win), 0;
-
        /* Reset focus after after focus change,
         * not sure what is causing the focus change in the first place
         * but preventing that would be a better solution */
@@ -880,7 +872,7 @@ void wm_insert(win_t *win)
 
        /* Arrange */
        wm_update();
-       set_focus(wm_focus);
+       set_focus(win);
        print_txt();
 }
 
@@ -910,8 +902,8 @@ void wm_init(win_t *root)
        wm->tag     = tag_new(wm->screens, 1);
        wm->tags    = list_insert(NULL, wm->tag);
 
-       event_t ev_e[] = {EV_ENTER, EV_FOCUS};
-       event_t ev_s[] = {'h', 'j', 'k', 'l', 'q', ' ',
+       event_t ev_e[] = {EV_ENTER, EV_FOCUS, EV_MOUSE1};
+       event_t ev_s[] = {'h', 'j', 'k', 'l', 'c', 'q', ' ',
                '0', '1', '2', '3', '4', '5', '6', '7', '8', '9'};
        event_t ev_m[] = {'h', 'j', 'k', 'l', 'd', 's', 'm', 't', ' ',
                '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
@@ -784,7 +779,7 @@ int wm_handle_event(win_t *win, event_t ev, mod_t mod, ptr_t ptr)
        }
 
-       /* Column mode commands */
+       /* Column layout commands */
        if (mod.MODKEY) {
                switch (ev) {